/*
 * This file implements the openssl BIO communication abstraction on top of my
 * own cleansocks abstraction.  It is derived from openssl bio/bss_sock.c (with
 * reference to bio/bss_fd.c) from the openssl source release.  Its
 * heading/copyright notice is reproduced exactly below.  Given how extensive
 * the changes are from the original code, it's at least questionable that it
 * is still a copy of the original, except for the interface requirements.
 * Which are probably not copyrightable unless the outragously greedy bastards
 * at Oracle finally prevail in their frightenly grasping Java suit against
 * Google and put an end to software innovation in the US.  In the mean time,
 * if you want to use this file, you'll have to figure out the rules for
 * yourself.  Since I don't know, I won't try to stop you.  If Mr.  Young
 * doesn't either, you're in the clear.  My fully original work (other files) is
 * released under the GPL.
 */

/*
 * This file is for openssl before version 1.1.0.  A different file is
 * needed for the later interface.
 */

/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
 * All rights reserved.
 *
 * This package is an SSL implementation written
 * by Eric Young (eay@cryptsoft.com).
 * The implementation was written so as to conform with Netscapes SSL.
 * 
 * This library is free for commercial and non-commercial use as long as
 * the following conditions are aheared to.  The following conditions
 * apply to all code found in this distribution, be it the RC4, RSA,
 * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
 * included with this distribution is covered by the same copyright terms
 * except that the holder is Tim Hudson (tjh@cryptsoft.com).
 * 
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.
 * If this package is used in a product, Eric Young should be given attribution
 * as the author of the parts of the library used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    "This product includes cryptographic software written by
 *     Eric Young (eay@cryptsoft.com)"
 *    The word 'cryptographic' can be left out if the rouines from the library
 *    being used are not cryptographic related :-).
 * 4. If you include any Windows specific code (or a derivative thereof) from 
 *    the apps directory (application code) you must include an acknowledgement:
 *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
 * 
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 */

#include <stdio.h>
#include <errno.h>

#include <string.h>
#include <errno.h>

#include "cleansri.h"
#include "cleansocks.h"

#include "cleanbio.h"

#define get_last_sys_error()	ERRCODE

inline cleansocks::sr_obj *sock(BIO *b)
{
	return static_cast<cleansocks::sr_obj *>(b->ptr);
}

struct clean_bio_methods {
public:
	static int write(BIO *h, const char *buf, int num);
	static int read(BIO *h, char *buf, int size);
	static long ctrl(BIO *h, int cmd, long arg1, void *arg2);
	static int init(BIO *h);
	static int free(BIO *data);
};

static BIO_METHOD methods_clean = {
	//50|BIO_TYPE_DESCRIPTOR|BIO_TYPE_SOURCE_SINK,
	50|BIO_TYPE_SOURCE_SINK, // Have no real idea.
	"cleansocks bio",
	clean_bio_methods::write,
	clean_bio_methods::read,
	NULL,
	NULL,
	clean_bio_methods::ctrl,
	clean_bio_methods::init,
	clean_bio_methods::free,
	NULL,
};

// Close flag just tells if the socket should be closed on free.
// Defined in openssl/bio.h
BIO *BIO_new_clean(cleansocks::sr_obj *s, int close_flag) {
	BIO *ret = BIO_new(&methods_clean);
	if(ret == NULL) return NULL;
	methods_clean.ctrl(ret,BIO_C_SET_FD,close_flag,s);
	return ret;
}

int clean_bio_methods::init(BIO *bi) {
	bi->init = 0;
	bi->num = -1;  // openssl sock has 0.
	bi->ptr = NULL;
	bi->flags = 0;
	return 1;
}

int clean_bio_methods::free(BIO *a) {
	if (a == NULL) return 0;
	if (a->shutdown) {
		if (a->init) {
			//::shutdown(a->num,2);  Don't have a shutdown.
			cleansocks::close(*sock(a));
		}
		a->init = 0;
		a->flags = 0;
        }
	return 1;
}
    
int clean_bio_methods::read(BIO *b, char *out, int outl) {
	int ret = 0;

	if(out != NULL) {
		BIO_clear_retry_flags(b);
		try {
			ret = cleansocks::recv(*sock(b),out,outl);
		} catch(cleansocks::socket_would_block &e) {
			BIO_set_retry_read(b);
			ret = -1;
		} catch(...) {
			ret = -1;
		}
        }
	return ret;
}

int clean_bio_methods::write(BIO *b, const char *in, int inl) {
	int ret;
	BIO_clear_retry_flags(b);
	try {
		ret = cleansocks::send(*sock(b),in,inl);
	} catch(cleansocks::socket_would_block &e) {
		BIO_set_retry_write(b);
		ret = -1;
	} catch(...) {
		ret = -1;
	}
	return ret;
}

long clean_bio_methods::ctrl(BIO *b, int cmd, long num, void *ptr) {
	long ret = 1;

	switch (cmd) {
	case BIO_C_SET_FD:
		free(b); // Note: definitely *NOT* ::free
		b->ptr = ptr;
		b->shutdown = (int)num;
		b->init = 1;
		break;
	case BIO_C_GET_FD:
		if (b->init) {
			if(ptr != NULL) {
				cleansocks::sr_obj **outptr = 
					(cleansocks::sr_obj **)ptr;
				*outptr = sock(b);
			} else
				ret = 0;
		}
		else
			ret = -1;
		break;
	case BIO_CTRL_GET_CLOSE:
		ret = b->shutdown;
		break;
	case BIO_CTRL_SET_CLOSE:
		b->shutdown = (int)num;
		break;
	case BIO_CTRL_DUP:
	case BIO_CTRL_FLUSH:
		ret = 1;
		break;
	default:
		ret = 0;
		break;
        }
	return ret;
}
