/*
 * This file implements the openssl BIO communication abstraction on top of my
 * own cleansocks abstraction.  This file is part of the Cleansocks library, 
 * It is released under the Lesser GNU GPL.  See the accompanying files COPYING
 * and COPYING.LESSER.
 */

/*
 * This file is for openssl version 1.1.0 and until the next time they change
 * the interface and break everything.  A different file is
 * needed for the older interface.
 */
#include <stdio.h>
#include <errno.h>

#include <string.h>
#include <errno.h>

#include "cleansri.h"
#include "cleansocks.h"

#include "cleanbio.h"

#include <openssl/bio.h>
#include <openssl/crypto.h>

struct clean_bio_methods {
public:
	static int write(BIO *h, const char *buf, int num);
	static int read(BIO *h, char *buf, int size);
	static long ctrl(BIO *h, int cmd, long arg1, void *arg2);
	static int init(BIO *h);
	static int free(BIO *data);
};

inline cleansocks::sr_obj *sock(BIO *b)
{
	return static_cast<cleansocks::sr_obj *>(BIO_get_data(b));
}

// Close flag just tells if the socket should be closed on free.
// Defined in openssl/bio.h
BIO *BIO_new_clean(cleansocks::sr_obj *s, int close_flag) {
	/* Prepare the methods dingus. */
	BIO_METHOD *bio =
		BIO_meth_new(50|BIO_TYPE_SOURCE_SINK, "cleansocks bio");
	BIO_meth_set_write(bio, clean_bio_methods::write);
	BIO_meth_set_read(bio, clean_bio_methods::read);
	BIO_meth_set_ctrl(bio, clean_bio_methods::ctrl);
	BIO_meth_set_create(bio, clean_bio_methods::init);
	BIO_meth_set_destroy(bio, clean_bio_methods::free);

	BIO *ret = BIO_new(bio);
	BIO_set_data(ret, s);
	return ret;
}

int clean_bio_methods::init(BIO *bi) {
	BIO_set_init(bi, 1);
	return 1;
}

int clean_bio_methods::free(BIO *a) {
	if (a == NULL) return 0;
	BIO_set_init(a, 0);
	return 1;
}
    
int clean_bio_methods::read(BIO *b, char *out, int outl) {
	int ret = 0;

	if(out != NULL) {
		BIO_clear_retry_flags(b);
		try {
			ret = cleansocks::recv(*sock(b),out,outl);
		} catch(cleansocks::socket_would_block &e) {
			BIO_set_retry_read(b);
			ret = -1;
		} catch(...) {
			ret = -1;
		}
        }
	return ret;
}

int clean_bio_methods::write(BIO *b, const char *in, int inl) {
	int ret;
	BIO_clear_retry_flags(b);
	try {
		ret = cleansocks::send(*sock(b),in,inl);
	} catch(cleansocks::socket_would_block &e) {
		BIO_set_retry_write(b);
		ret = -1;
	} catch(...) {
		ret = -1;
	}
	return ret;
}

long clean_bio_methods::ctrl(BIO *b, int cmd, long num, void *ptr) {
	switch (cmd) {
	case BIO_CTRL_FLUSH:
		// Seems to need to do this, even though there's nothing
		// to do.  So here we are.  If the buffering socket ever
		// buffers output, we may need to do something here.
		return 1;
	}
	return 0;
}
