/*
 * Run with a port number.  Listens for a connection and copies 
 * to stdout.
 */

	
/*
 * This file is part of the Cleansocks library.  It is released under the
 * Lesser GNU GPL.  See the accompanying files COPYING and COPYING.LESSER.
 */

#include <stdlib.h>

#include <iostream>
#include <string>
#include <stdexcept>

#include "cleansocks.h"
#include "cleanip.h"

using namespace cleansocks;

void doit(const IPport &p)
{
	TCPsocket listner;
	bind(listner, IPendpoint(IPaddress::any(), p));
	listen(listner);
	while(true) {
		std::cout << "Waiting..." << std::endl;
		std::cout.flush();
#ifdef NOFROM
		TCPsocket s = accept(listner);
		std::cout << "accepted connection" << std::endl;
#else
		IPendpoint from;
		TCPsocket s = accept(listner, from);
		std::cout << "accepted connection from " << from << std::endl;
#endif
		// Copy it out.
		int totbytes = 0;
		char buf[100];
		int n;
		while((n = recv(s, buf, sizeof buf)) != 0) {
			std::cout.write(buf, n);
			totbytes += n;
		}
		close(s);

		std::cout << "Connection closed.  " << totbytes 
			  << " copied." << std::endl;

		if(totbytes == 0) break;
	}

	close(listner);
}

int main(int argc, char **argv)
{
	if(argc != 2) {
		std::cerr << "Provide exactly one port number." << std::endl;
		exit(2);
	}

	IPport p = atoi(argv[1]);

	try {
		doit(p);
	} catch(socket_db_error &e) {
		std::cout << "Socket db error: " << e.what() << std::endl;
	}
	catch(socket_error &e) {
		std::cout << "Socket error: " << e.what() << std::endl;
	}
	catch(std::runtime_error &e) {
		std::cout << "Runtime error: " << e.what() << std::endl;
	}
	
}
